'use strict';

var shim = require('use-sync-external-store/shim');

var run = function (fn) {
    fn();
};
var __DEV__ = process.env.NODE_ENV !== 'production';
var resso = function (state) {
    if (__DEV__ && Object.prototype.toString.call(state) !== '[object Object]') {
        throw new Error('object required');
    }
    var store = {};
    Object.keys(state).forEach(function (key) {
        if (typeof state[key] === 'function') {
            return;
        }
        var listeners = new Set();
        store[key] = {
            subscribe: function (listener) {
                listeners.add(listener);
                return function () { return listeners.delete(listener); };
            },
            getSnapshot: function () { return state[key]; },
            setSnapshot: function (val) {
                if (val !== state[key]) {
                    state[key] = val;
                    run(function () { return listeners.forEach(function (listener) { return listener(); }); });
                }
            },
            useSnapshot: function () {
                return shim.useSyncExternalStore(store[key].subscribe, store[key].getSnapshot);
            },
        };
    });
    return new Proxy(state, {
        get: function (_, key) {
            try {
                return store[key].useSnapshot();
            }
            catch (e) {
                return state[key];
            }
        },
        set: function (_, key, val) {
            store[key].setSnapshot(val);
            return true;
        },
    });
};
resso.config = function (_a) {
    var batch = _a.batch;
    run = batch;
};

module.exports = resso;
